@@ -13,7 +13,7 @@ from maslourl.trackers.average_tracker import AverageRewardTracker
 from abc import ABC, abstractmethod
 
 
-class MaslouTorchAgent(nn.Module, ABC):
+class MaslouTorchModel(nn.Module, ABC):
     @abstractmethod
     def get_value(self, x):
         pass
@@ -26,39 +26,44 @@ class MaslouTorchAgent(nn.Module, ABC):
     def get_action_greedy(self, x):
         pass
 
+class MaslouPPOAgent(ABC):
 
-class MaslouPPO(ABC):
+    def __init__(self, args=None, discrete=True, eval=False):
+        if not eval and args is None:
+            raise ValueError("If agent is in train state then u nust give args, else put eval=True")
+        self.eval = eval
+        if not self.eval:
+            self.args = args
+            self.discrete=discrete
+            self.run_name = f"{args.gym_id}_{args.exp_name}_{args.seed}_{int(time.time())}"
+            if args.track:
+                import wandb
 
-    def __init__(self, args, discrete=True):
-        self.args = args
-        self.discrete=discrete
-        self.run_name = f"{args.gym_id}_{args.exp_name}_{args.seed}_{int(time.time())}"
-        if args.track:
-            import wandb
+                wandb.init(project=args.wandb_project_name,
+                           entity=args.wandb_entity,
+                           sync_tensorboard=True,
+                           config=vars(args),
+                           name=self.run_name,
+                           monitor_gym=True,
+                           save_code=True)
 
-            wandb.init(project=args.wandb_project_name,
-                       entity=args.wandb_entity,
-                       sync_tensorboard=True,
-                       config=vars(args),
-                       name=self.run_name,
-                       monitor_gym=True,
-                       save_code=True)
-
-        self.writer = SummaryWriter(f"runs/{self.run_name}")
-        self.writer.add_text("hyperparameters",
-                             "|param|value|\n|-|-|\n%s" % (
-                                 "\n".join([f"|{key}|{value}" for key, value in vars(args).items()])))
-        random.seed(args.seed)
-        np.random.seed(args.seed)
-        torch.manual_seed(args.seed)
-        torch.backends.cudnn.deterministic = args.torch_deterministic
-        self.device = torch.device("cuda" if torch.cuda.is_available() and args.cuda else "cpu")
-        self.envs = gym.vector.SyncVectorEnv(
-            [self.make_env(args.gym_id, args.seed + i, i, args.capture_video, args.capture_every_n_video, self.run_name)
-             for i in
-             range(args.num_envs)])
-        if isinstance(self.envs.single_action_space, gym.spaces.Discrete) != self.discrete:
-            raise f"Illegal type of action. You want {'discrete' if self.discrete else 'continuous'}, but gym want {self.envs.single_action_space}"
+            self.writer = SummaryWriter(f"runs/{self.run_name}")
+            self.writer.add_text("hyperparameters",
+                                 "|param|value|\n|-|-|\n%s" % (
+                                     "\n".join([f"|{key}|{value}" for key, value in vars(args).items()])))
+            random.seed(args.seed)
+            np.random.seed(args.seed)
+            torch.manual_seed(args.seed)
+            torch.backends.cudnn.deterministic = args.torch_deterministic
+            self.device = torch.device("cuda" if torch.cuda.is_available() and args.cuda else "cpu")
+            self.envs = gym.vector.SyncVectorEnv(
+                [self.make_env(args.gym_id, args.seed + i, i, args.capture_video, args.capture_every_n_video, self.run_name)
+                 for i in
+                 range(args.num_envs)])
+            if isinstance(self.envs.single_action_space, gym.spaces.Discrete) != self.discrete:
+                raise f"Illegal type of action. You want {'discrete' if self.discrete else 'continuous'}, but gym want {self.envs.single_action_space}"
+        else:
+            self.env = self.make_env("", 42, 0, False, 0, "test")
         self.agent = self.build_agent().to(self.device)
 
     @abstractmethod
@@ -66,28 +71,30 @@ class MaslouPPO(ABC):
         pass
 
     @abstractmethod
-    def build_agent(self) -> MaslouTorchAgent:
+    def build_agent(self) -> MaslouTorchModel:
         pass
 
     def save_agent(self):
         if not os.path.exists("./models/"):
             os.makedirs("./models/")
-        torch.save(self.agent, "models/best_model.pt")
+        torch.save(self.agent, "models/model.pt")
         if self.args.track and self.args.save_best_to_wandb:
             if self.args.verbose:
-                print("Saving file to w&b", os.path.join(wandb.run.dir, "models/best_model.pt"))
+                print("Saving file to w&b", os.path.join(wandb.run.dir, "models/model.pt"))
             if not os.path.exists(os.path.join(wandb.run.dir, "models/")):
                 os.makedirs(os.path.join(wandb.run.dir, "models/"))
-            torch.save(self.agent, os.path.join(wandb.run.dir, "models/best_model.pt"))
+            torch.save(self.agent, os.path.join(wandb.run.dir, "models/model.pt"))
 
-    def load_agent(self, path, train=False):
+    def load_agent(self, path):
         self.agent = torch.load(path)
-        if train:
+        if not self.eval:
             self.agent.train()
         else:
             self.agent.eval()
 
     def train(self):
+        if self.eval:
+            raise ValueError("The agent is in eval state")
         average_reward_tracker = AverageRewardTracker(self.args.average_reward_2_save)
         best_rewards = -np.inf
         optimizer = optim.Adam(self.agent.parameters(), lr=self.args.learning_rate, eps=1e-5)
